package Backpropagation;

import java.util.Random;
import java.util.Vector;

//Neuron in the backpropagation network. 
public class Element implements Cloneable{
	//Data
	public double y;
	public double delta;
	public Vector<Double> last_delta_weight; // used for momentum
	public Vector<Double> weight;
	
	public Element(){
		this.y=0.0;
		this.delta=0.0;
		this.weight = null;
		this.last_delta_weight = null;
	}
	
	public Element(int dimension){
		this.weight = new Vector<Double>();
		this.weight.setSize(dimension);
		
		this.last_delta_weight = new Vector<Double>();
		this.last_delta_weight.setSize(dimension);
		
		//Done in reset weights
		//for(int j=0 ; j<this.last_delta_weight.size() ; j++)
		//	this.last_delta_weight.set(j, 0.0);
		
		this.resetWeights();
	}
	//Copy constructor -- I don`t know an easier way how to copy complex structures in java....
	public Element(Element other){
		this.y = other.y;
		this.delta = other.delta;
		if(other.last_delta_weight != null){
			this.last_delta_weight = new Vector<Double>();
			this.last_delta_weight.setSize(other.last_delta_weight.size());
		}
		if(other.weight != null){
			this.weight = new Vector<Double>();
			this.weight.setSize(other.weight.size());
			for(int i=0; i<other.weight.size() ; i++){
				this.weight.set(i, new Double(other.weight.get(i)));
			}
		}
	}
	
	public double getWeightAt(int index){
		if(this.weight == null)
			return 0.0;
		return this.weight.get(index);
	}
	
	//Sets weights to random. 
	public void resetWeights(){
		if(this.weight == null)
			return;
		
		Random r = new Random();
		for(int i=0; i<this.weight.size() ; i++){
			//Random weight around 0.0 with parabolic distribution
			this.weight.set(i, (r.nextDouble()*2.0 - 1.0)*(r.nextDouble()*2.0 - 1.0));
		}
		//Reset last delta weights. 
		for(int j=0 ; j<this.last_delta_weight.size() ; j++)
			this.last_delta_weight.set(j, 0.0);
	}
	
	//Computes output of this neuron and stores it into this.y
	public void computeY(Layer input_layer){
		if(this.weight == null) //Input and Threshold neurons
			return;
		
		double net = 0.0; 
		for(int i=0; i<this.weight.size() ; i++){
			net += this.weight.get(i) *  input_layer.get(i).y;
		}
		
		this.y = Scout.activationFunction(net);
	}
	
	//Delta rule for the output neuron
	public void computeDelta(double desired_output){
		this.delta = (desired_output - this.y) * Scout.derivationOfActivationFunction(this.y);
	}
	
	//Delta rule for the hidden layer neuron. 
	public void computeDelta(Layer output_layer, int this_index){
		if(this.weight == null) //Input and Threshold neurons
			return;
			
		this.delta = 0.0; 
		for(int i=0; i < output_layer.size() ; i++){
			//Threshold is not counted. 
			this.delta += output_layer.get(i).delta 
						  * output_layer.get(i).getWeightAt(this_index);
		}
		this.delta *= Scout.derivationOfActivationFunction(this.y);
	}
	
	//Learning rule (hidden and output neurons)
	public void learn(Layer input_layer, double alpha, double mi){
		if(this.weight == null) //Input and Threshold neurons
			return;
		
		//w_ij update
		for(int j=0 ; j < this.weight.size() ; j++){
			double delta_w = alpha * this.delta * input_layer.get(j).y;
			this.weight.set(j, 
				//alpha * delta_i * input_j + momentum * last_delta_weight
				this.weight.get(j) + delta_w + mi * this.last_delta_weight.get(j)
			);
			this.last_delta_weight.set(j, delta_w);
		}
	}
}
