package Backpropagation;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Vector;

//Controller class for finding good models. 
public class Scout {
	/**
	 * @param args
	 */
	//Output to file -- it will be serialized for plots. 
	public static FileOutputStream out; 
	public static PrintStream log;
	
	public static void main(String[] args) {
		try {
			out = new FileOutputStream("log.log");
			log = new PrintStream(out);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//Constants
		int input_dimension = 2;
		int output_dimension = 3; 
		int input_row_count = 800;
		String input_file = "2d.trn.dat";
		String filedir = "E:\\programming\\java_eclipse_workspace\\NeuralNetworks\\data\\";
		
		//Load train data
		Supervisor supervisor = new Supervisor(input_dimension, output_dimension);
		supervisor.addTrainingEntriesFromFile(filedir + input_file, input_row_count);
		
		//Get cases which will be tested
		Vector<CaseData> cases = Scout.initCases();
		
		//Print how many are of them 
		System.out.println("Now I will try "+((Integer)cases.size())+" models.");
		
	
		//Go thru all constant sets
		for(int cid = 0 ; cid < cases.size() ; cid++){
			System.out.println("CID:" + ((Integer)cid).toString() + " out of " + ((Integer)cases.size()));
			CaseData data = cases.get(cid);
			
			//Prepare the inputs
			supervisor.permutateRandom();
			
			//Initialize network
			Backpropagation B = new Backpropagation(
					input_dimension,
					output_dimension,
					data.layer_sizes);
			B.setAlpha(data.alpha);
			B.setMomentum(data.mi);
			Scout.sigmoid_tangent = data.sigmoid_tangent;
			
			//Train with k-cross validation, remembers model instances, prints errors to log 
			data.cv = B.trainNetwork(supervisor, data.max_epoch_count, data.k_fold, false);
			data.backprogagation = B;
			//Summarize this test case
			data.printLine(true);
		}
		
		//Find the best model
		CaseData best_cd = null;
		for(int cid = 0; cid < cases.size() ; cid++){
			if(best_cd == null || cases.get(cid).cv < best_cd.cv){
				best_cd = cases.get(cid);
			}
		}
		
		System.out.println("=====================Best: ");
		best_cd.print();
		
		Backpropagation bestBP = best_cd.backprogagation;
		bestBP.setBestInstance();
		Scout.sigmoid_tangent = best_cd.sigmoid_tangent;
		
		input_row_count = 200;
		input_file = "2d.tst.dat";
		Supervisor tester = new Supervisor(input_dimension, output_dimension);
		tester.addTrainingEntriesFromFile(filedir + input_file, input_row_count);
		
		//Test the best model on test data 
		double tst_error = bestBP.testNetwork(tester, false);
		//Print classifications and also the confusion matrix. 
		bestBP.testClassification(tester, true, true);
		bestBP.printNetwork();
		
		System.out.println("Error on test: " + ((Double)tst_error));
		
		//Show the progress of errors (est, val) on the best model. 
		bestBP.resetNetwork();
		bestBP.trainNetwork(1000, supervisor.extractComplement(0, 1, 10), supervisor.extract(0, 1, 10), true);
		//Test it 
	}

	
	//false - sigmoid, true - tangent
	public static boolean sigmoid_tangent = false; 

	//Input -- net
	//-c*net (c -- temperature of the network)
	public static double activationFunction(double net){
		if(Scout.sigmoid_tangent)
			return Math.tanh(net);
		else
			return 1.0 / (1.0 + Math.pow(Math.E, -net));
	}

	//Input -- y
	public static double derivationOfActivationFunction(double _y){
		if(Scout.sigmoid_tangent)
			return (1+_y)*(1-_y);
		else
			return _y*(1-_y);
	}
	
	public static Vector<CaseData> initCases(){
		/*
	public int max_epoch_count = 1000;
	public double alpha = 0.15;
	public double mi = 0.5;
	public int layer_count = 1;
	public int k_fold = 10;
	public Vector<Integer> layer_sizes = new Vector<Integer>();
	public boolean sigmoid_tangent = false;
		 * */
		Vector<Double> alphas = new Vector<Double>();
		alphas.add(0.05);
		alphas.add(0.1);
		alphas.add(0.20);
		alphas.add(0.30);
		//alphas.add(0.5);
		Vector<Double> mis = new Vector<Double>();
		mis.add(0.0);
		mis.add(0.1);
		mis.add(0.3);
		mis.add(0.5);
		Vector<Vector<Integer>> layer_sizes = new Vector<Vector<Integer>>();
		Vector<Integer> layer1 = new Vector<Integer>();
		layer1.add(16);
		layer1.add(32);
		layer1.add(48);
		int layer_count = 2;
		for(int k=1; k<=layer_count ; k++){
			for(int i=0; i<layer1.size(); i++){
				Vector<Integer> V = new Vector<Integer>();
				for(int a=0; a<k; a++){
					V.add((int)(layer1.get(a)));
				}
				layer_sizes.add(V);
			}
		}
		
		
		Vector<Boolean> activationFs = new Vector<Boolean>();
		activationFs.add(false);
		//activationFs.add(true);

		//Each variation of the considerable values of main constants.
		Vector<CaseData> cases = new Vector<CaseData>();
		for(int a=0; a < alphas.size() ; a++){
			for(int m=0; m < mis.size() ; m++){
				for(int ls=0; ls < layer_sizes.size() ; ls++){
					for(int b=0; b < activationFs.size() ; b++){
						CaseData data = new CaseData();
						data.alpha = alphas.get(a);
						data.backprogagation = null;
						data.cv = 132456798.0;
						data.k_fold = 10;
						data.layer_sizes = layer_sizes.get(ls);
						data.max_epoch_count = 2; //!Test only!
						data.mi = mis.get(m);
						data.sigmoid_tangent = activationFs.get(b);
						cases.add(data);
					}
				}
			}
		}
		
		return cases;
	}
}
