package Backpropagation;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Random;
import java.util.Scanner;
import java.util.Vector;

//Represents the input-output data sets. 
//---implements basic operations with them. 
public class Supervisor {
	private int current_set_id;
	private Vector<TrainingEntry> set; 
	
	private int input_dimension;
	private int output_dimension; 
	
	//Creates a subset of the original supervisor. 
	private Supervisor(Supervisor supervisor, int start_index, int end_index){
		this(supervisor.input_dimension, supervisor.output_dimension);
		
		if(end_index > start_index){
			this.set.setSize(end_index - start_index);
			for(int i=0; i < end_index - start_index ; i++){
				this.set.set(i, supervisor.set.get(start_index + i));
			}
		}
	}
	
	//Specifies the input-output dimensions 
	public Supervisor(int _input_dimension, int _output_dimension){
		this.set = new Vector<TrainingEntry>();
		this.current_set_id = 0;
		
		//!Todo // make sure these constraints are well applied
		this.input_dimension = _input_dimension;
		this.output_dimension = _output_dimension;
	}

	public int getSize(){
		return this.set.size();
	}
	
	public int getInputDimension(){
		return this.input_dimension;
	}
	public int getOutputDimension(){
		return this.output_dimension;
	}
	
	public void addTrainingEntry(TrainingEntry _new){
		this.set.add(_new);
	}
	
	public TrainingEntry getCurrent(){
		return this.set.get(this.current_set_id);
	}
	public double getInputByIndex(int index){
		return this.getCurrent().input.get(index);
	}
	public double getOutputByIndex(int index){
		return this.getCurrent().output.get(index);
	}
	
	//Permutates the TrainingEntries randomly
	public void permutateRandom(){
		Random r = new Random();
		for(int i=0; i<this.set.size() - 1 ; i++){
			int j = i + 1 + (r.nextInt(this.set.size() - i - 1));
			TrainingEntry tv = this.set.get(i);
			this.set.set(i, this.set.get(j));
			this.set.set(j, tv);
		}
	}
	
	//9-1 cross validation will be 0,9,10 and 9,10,10
	public Supervisor extract(int from, int to, int parts){
		int a = (this.set.size() * from) / parts;
		int b = ((this.set.size() * to) / parts) - 1;
		
		return new Supervisor(this, a, b);
	}
	//extract united extractComplement is the whole (extract intersection extractComplement is emptySet)
	public Supervisor extractComplement(int from, int to, int parts){
		int a = (this.set.size() * from) / parts;
		int b = ((this.set.size() * to) / parts) - 1;
		
		Supervisor result = new Supervisor(this, 0, a);
		for(int i = b ; i < this.set.size() ; i++){
			result.set.add(this.set.get(i));
		}
		return result;
	}
	
	/*File structure : M-rows, N-dimension
	 * in a row there are input_dimension+1 (for class) values
	 * */
	public void addTrainingEntriesFromFile(String filename, int row_count){
		FileReader fr = null;
		try {
			fr = new FileReader(filename);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}
		Scanner s = new Scanner(fr);
		
		for(int i=0; i<row_count; i++){
			TrainingEntry te = new TrainingEntry();
			te.input.setSize(input_dimension);
			for(int j=0; j<input_dimension; j++){
				te.input.set(j, s.nextDouble());
			}
			te.output = this.parseOutput(s);
			
			this.addTrainingEntry(te);
		}
	}
	
	//!Refactor --- as a parameter
	//--class to vector
	public Vector<Double> parseOutput(Scanner s){
		Vector<Double> result = new Vector<Double>();
		result.setSize(3);
		
		int i = (int)s.nextInt();
		//System.out.println(i);
		switch(i)
		{
		case 0: 
			result.set(0, 1.0);
			result.set(1, 0.0);
			result.set(2, 0.0);
		break;
		case 1: 
			result.set(0, 0.0);
			result.set(1, 1.0);
			result.set(2, 0.0);
		break;
		case 2: 
			result.set(0, 0.0);
			result.set(1, 0.0);
			result.set(2, 1.0);
		break;
		}
		return result;
	}

	//!Refactor --- as a parameter 
	//--vector to class
	public int classify(Vector<Double> output){
		double max = -123456789.0;
		int result = 0;
		for(int i=0; i<output.size() ; i++){
			if(output.get(i) > max){
				max = output.get(i);
				result = i; 
			}
		}
		return result;
	}

	public boolean next(){
		this.current_set_id++;
		if(this.current_set_id == this.set.size()){
			this.permutateRandom();
			this.current_set_id = 0;
			return false;
		}
		return true;
	}
	
	public void printEntries(){
		for(int i=0; i<this.set.size() ; i++){
			TrainingEntry te = this.set.get(i);
			for(int j=0; j<te.input.size() ; j++){
				System.out.print(((Double)te.input.get(j)).toString() + " ");
			}
			System.out.print("|| ");
			for(int j=0; j<te.output.size() ; j++){
				System.out.print(((Double)te.output.get(j)).toString() + " ");
			}
			System.out.println();
		}
	}
}
